package enumerators

import (
	"log"

	gitea "code.gitea.io/sdk/gitea"
)

func convertGiteaRepoToEndpointData(mode CloneMethod, repo *gitea.Repository) *RepositoryEndpoint {
	var CloneURL string
	switch mode {
	case SSHClone:
		CloneURL = repo.SSHURL
	case HTTPClone:
		CloneURL = repo.CloneURL
	}
	return &RepositoryEndpoint{
		Url:      CloneURL,
		FullName: repo.FullName,
		Push:     repo.Permissions.Push && !repo.Mirror,
		Pull:     repo.Permissions.Pull,
	}
}

func CreateGiteaEnumerator(config *EnumeratorConfig) (Enumerator, error) {
	client, err := gitea.NewClient(config.ApiUrl, gitea.SetBasicAuth(config.Username, config.Password))
	if err != nil {
		return nil, err
	}

	return func(repoChannel chan *RepositoryEndpoint) error {
		orgs, _, err := client.ListMyOrgs(gitea.ListOrgsOptions{ListOptions: gitea.ListOptions{Page: -1}})
		if err != nil {
			return err
		}

		for _, org := range orgs {
			orgRepos, _, err := client.ListOrgRepos(org.UserName, gitea.ListOrgReposOptions{
				ListOptions: gitea.ListOptions{Page: -1},
			})
			if err != nil {
				return err
			}
			for _, repo := range orgRepos {
				repoChannel <- convertGiteaRepoToEndpointData(config.CloneMethod, repo)
			}
		}

		repos, _, err := client.ListMyRepos(gitea.ListReposOptions{ListOptions: gitea.ListOptions{Page: -1}})
		log.Println("Got repos")
		if err != nil {
			return err
		}

		for _, repo := range repos {
			repoChannel <- convertGiteaRepoToEndpointData(config.CloneMethod, repo)
		}

		return nil
	}, nil
}
